//
//  ImagePanel.java
//  CBSim
//
//  Created by Loren Petrich on 5/18/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;

class ImagePanel extends JPanel
{
	// Original and result images
	BufferedImage OrigImg, ResImg;
	
	// Shows the full filespec or URL
	private JLabel TitleDisplay;
	
	// Names of the possibilities:
	static final String OptionNames[] = {"Normal",
		"No L (Red) - Protanopia",
		"No M (Green) - Deuteranopia",
		"No S (Blue) - Tritanopia",
		"Dog Vision", "Gray - Brightness",
		"Gray - L (Red)", "Gray - M (Green)", "Gray - S (Blue)"};
	
	// How many
	static final int NumTypes = OptionNames.length;
		
	// What we see
	private JLabel ImageDisplay;
	private ListenerSetType TypeListener;
	
	private JToggleButton[] TypeButtons = new JToggleButton[NumTypes];
	
	// Partial and gamma dialog box:
	private final ImageExtrasDialog Extras = new ImageExtrasDialog();
	
	private static final String Lbl_Colors = "Colors";
	private static final String Lbl_Grays = "Grays";
	
	ImagePanel(BufferedImage Img, ListenerSetType TypeListener_, String Title)
	{
		ResImg = OrigImg = Img;
		TypeListener = TypeListener_;
		
		setLayout(new BorderLayout());
		
		// Set up the toolbar
		JPanel Toolbar = new JPanel();
		
		// Divide the icons into tabbed panes
		final JPanel TypeButtonPaneSet = new JPanel(new CardLayout());
		
		JPanel ColorPane = new JPanel();
		TypeButtonPaneSet.add(ColorPane, Lbl_Colors);
		
		JPanel GrayPane = new JPanel();
		TypeButtonPaneSet.add(GrayPane, Lbl_Grays);
		
		// Use radio buttons to select which ones
		
		JRadioButton ColorButton = new JRadioButton("Colors");
		ColorButton.setActionCommand(Lbl_Colors);
		ColorButton.setSelected(true);
		
		JRadioButton GrayButton = new JRadioButton("Grays");
		GrayButton.setActionCommand(Lbl_Grays);
		
		ButtonGroup CGButtonLinking = new ButtonGroup();
		CGButtonLinking.add(ColorButton);
		CGButtonLinking.add(GrayButton);
		
		JPanel CGButtonSet = new JPanel(new GridLayout(0, 1));
		CGButtonSet.add(ColorButton);
		CGButtonSet.add(GrayButton);
		
		ActionListener CGSel = new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				CardLayout cl = (CardLayout)(TypeButtonPaneSet.getLayout());
				cl.show(TypeButtonPaneSet, (String)e.getActionCommand());
			}
		};
		
		ColorButton.addActionListener(CGSel);
		GrayButton.addActionListener(CGSel);
		
		// Add them in order
		Toolbar.add(CGButtonSet);
		Toolbar.add(TypeButtonPaneSet);
		
		// Type buttons are mutually exclusive
		ButtonGroup grp = new ButtonGroup();
		
		ActionListener BL = new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				TypeButtonPressed();
			}
		};
		
		Class cls = this.getClass();
		
		ImageIcon TypeIcons[] = new ImageIcon[2];
		
		for (int it=0; it<NumTypes; it++)
		{
			for (int isl=0; isl<2; isl++)
			{
				String filename = String.format("TypeIcon_%d_%d.png",it,isl);
				ImageIcon TypeIcon = new ImageIcon(cls.getResource(filename));
				TypeIcons[isl] = TypeIcon;
			}
			JToggleButton TypeButton = new JToggleButton(TypeIcons[0]);
			TypeButton.setSelectedIcon(TypeIcons[1]);
			TypeButton.setToolTipText(OptionNames[it]);
			
			if (it == 0) TypeButton.setSelected(true);
			TypeButton.addActionListener(BL);
			TypeButtons[it] = TypeButton;
			grp.add(TypeButton);
			
			JPanel Dest = null;
			if (it <5)
				Dest = ColorPane;
			else
				Dest = GrayPane;
			
			Dest.add(TypeButton);
		}
		
		JButton ExtrasButton = new JButton("Extras");
		ExtrasButton.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					ShowExtrasDialog();
				}
			}
		);
		Toolbar.add(ExtrasButton);
						
		// Add the title
		TitleDisplay = new JLabel(Title);
		TitleDisplay.setToolTipText("Full title (filename or URL)");
		TitleDisplay.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		JPanel TitleToolbar = new JPanel();
		TitleToolbar.setLayout(new BoxLayout(TitleToolbar, BoxLayout.Y_AXIS));
		TitleToolbar.add(TitleDisplay);		
		TitleToolbar.add(Toolbar);

		add(TitleToolbar, BorderLayout.NORTH);
		
		// Set up the image display
		// Ensure that if there is not enough space to display the image,
		// then the image will get scrollbars for scrolling through it
		ImageDisplay = new JLabel(new ImageIcon(ResImg));
		JScrollPane ImDispCtnr = new JScrollPane(ImageDisplay,
			ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		add(ImDispCtnr, BorderLayout.CENTER);
	}
	
	String GetTitle()
	{
		return TitleDisplay.getText();
	}
	
	private void TypeButtonPressed()
	{
		UpdateImage(GetType());
	}
	
	int GetType()
	{
		int whch = 0;
		for (int it=0; it<NumTypes; it++)
		{
			JToggleButton TypeButton = TypeButtons[it];
			if (TypeButton.isSelected()) whch = it;
		}
		TypeListener.SetType(whch);
		return whch;
	}
	
	void SetType(int whch)
	{
		boolean oldsel = TypeButtons[whch].isSelected();
		TypeButtons[whch].setSelected(true);
			if (!oldsel) UpdateImage(whch);
	}
	
	void UpdateImage(int whch)
	{
		// For convenience
		CB_Filter fltr = null;
		
		if (whch == 0)
			ResImg = OrigImg;
		else if (whch <= 4)
		{
			// Set up the color-blindness filter
			switch(Extras.algo)
			{
				// Reverse-engineered linear filter
				case 0: fltr = new CB_Filter_Linear(); break;
				// Brettel-Vienot-Mollot
				case 1: fltr = new CB_Filter_BVM(); break;
				// Meyer-Greenberg-Wolfmaier-Wickline
				case 2: fltr = new CB_Filter_MGWW(); break;
			}
			fltr.setType(whch-1);			
		}
		else
		{
			// Set up the grayscale filter
			fltr = new CB_Filter_Grayscale();
			fltr.setType(whch-5);
		}
		
		if (whch != 0)
		{
			fltr.fraction = Extras.fraction;
			fltr.setGamma(Extras.gamma);
			fltr.off_edge = Extras.off_edge;
			
			setCursor(new Cursor(Cursor.WAIT_CURSOR));
			ResImg = fltr.filter(OrigImg);
			setCursor(null);
		}
		
		ImageDisplay.setIcon(new ImageIcon(ResImg));
	}
	
	void ShowExtrasDialog()
	{
		if (Extras.show())
			UpdateImage(GetType());
	}
}
